# indoxRag/__init__.py
# from .indoxRag import IndoxRetrievalAugmentation

# from .cag import CAG

# import importlib.metadata

# __version__ = importlib.metadata.version("indoxRag")
__version__ = "0.0.14"
__author__ = "osllm.ai"
__license__ = "AGPL-3.0"
__copyright__ = "Copyright (c) 2024 osllm.ai"
